@@ -99,7 +99,15 @@ class _ConsoleHandler(object):
             except KeyError:
                 pass
             if logvalue in ('full', ''):
-                self._alwayson()
+                # if the *only* thing to change is the log,
+                # then let always on handle reconnect if needed,
+                # since we want to avoid a senseless disconnect
+                # if already connected
+                # if other things change, then unconditionally reconnect
+                onlylogging = len(nodeattribs[self.node]) == 1
+                self._alwayson(doconnect=onlylogging)
+                if onlylogging:
+                    return
             else:
                 self._ondemand()
                 if logvalue == 'none':
@@ -112,8 +120,10 @@ class _ConsoleHandler(object):
             return
         self.logger.log(*args, **kwargs)
 
-    def _alwayson(self):
+    def _alwayson(self, doconnect=True):
         self._isondemand = False
+        if not doconnect:
+            return
         if not self._console and not self.connectionthread:
             self._connect()
         else:
@@ -286,6 +296,9 @@ class _ConsoleHandler(object):
             logdata=username, ltype=log.DataTypes.event,
             event=log.Events.clientdisconnect, eventdata=edata)
 
+    def reopen(self):
+        self._got_disconnected()
+
     def _handle_console_output(self, data):
         if type(data) == int:
             if data == conapi.ConsoleEvent.Disconnect:
@@ -415,6 +428,8 @@ class ConsoleSession(object):
 
     def __init__(self, node, configmanager, username, datacallback=None):
         self.tenant = configmanager.tenant
+        if not configmanager.is_node(node):
+            raise exc.NotFoundException("Invalid node")
         consk = (node, self.tenant)
         self.ckey = consk
         self.username = username
@@ -438,6 +453,9 @@ class ConsoleSession(object):
     def send_break(self):
         self.conshdl.send_break()
 
+    def reopen(self):
+        self.conshdl.reopen()
+
     def destroy(self):
         _handled_consoles[self.ckey].detachuser(self.username)
         _handled_consoles[self.ckey].unregister_rcpt(self.reghdl)
