@@ -128,7 +128,6 @@ def send_response(responses, connection):
 
 
 def process_request(connection, request, cfm, authdata, authname, skipauth):
-    #TODO(jbjohnso): authorize each request
     if not isinstance(request, dict):
         raise ValueError
     operation = request['operation']
@@ -173,6 +172,9 @@ def process_request(connection, request, cfm, authdata, authname, skipauth):
                     elif data['operation'] == 'break':
                         consession.send_break()
                         continue
+                    elif data['operation'] == 'reopen':
+                        consession.reopen()
+                        continue
                     else:
                         raise Exception("TODO")
                 if not data:
